#include <qapplication.h>
#include <qclipboard.h>
#include <qmultilinedit.h>
#include <qpopupmenu.h>

class ClipboardMultiLineEdit : public QMultiLineEdit
{
  Q_OBJECT

public:
  ClipboardMultiLineEdit( QWidget* parent = 0, const char* name = 0 );

protected:
  void mouseReleaseEvent( QMouseEvent* event );

private slots:
  void clipboardChanged();

private:
  QPopupMenu* _menu;
  QClipboard* _clipboard;
  int _pasteitem;
};


ClipboardMultiLineEdit::ClipboardMultiLineEdit( QWidget* parent, 
												const char* name ) :
  QMultiLineEdit( parent, name )
{
  _menu = new QPopupMenu();
  _menu->insertItem( "Cut", this, SLOT( cut() ) );
  _menu->insertItem( "Copy", this, SLOT( copyText() ) );
  _pasteitem = _menu->insertItem( "Paste", this, SLOT( paste() ) );
  _menu->setItemEnabled( _pasteitem, false );

  _clipboard = QApplication::clipboard();
  connect( _clipboard, SIGNAL( dataChanged() ), 
		   this, SLOT( clipboardChanged() ) );
}


void ClipboardMultiLineEdit::mouseReleaseEvent( QMouseEvent* event )
{
  if( event->button() == RightButton )
	_menu->popup( mapToGlobal( event->pos() ) );
}


void ClipboardMultiLineEdit::clipboardChanged()
{
  if( strlen( _clipboard->text() ) )
	_menu->setItemEnabled( _pasteitem, true );
  else
	_menu->setItemEnabled( _pasteitem, false );
}
  

#include "clipboard2.moc"

int main( int argc, char* argv[] )
{
  QApplication a( argc, argv );
  
  ClipboardMultiLineEdit* edit = new ClipboardMultiLineEdit();
  a.setMainWidget( edit );
  edit->show();

  return a.exec();
}
