#include <qapplication.h>
#include <qcheckbox.h>
#include <qdialog.h>
#include <qpushbutton.h>

/**
  * This is our dialog. Note that you cannot see from the class
  * declaration whether this is a modal dialog or not.
  */
class MyDialog : public QDialog
{
  Q_OBJECT

public: 
  MyDialog();

  bool isCheckBoxChecked() const { return _checkbox->isChecked(); }

private:
  QCheckBox* _checkbox;
};
  

/**
  * The constructor for our dialog. By passing "true" as the third
  * parameter to the constructor of the base class, QDialog, we make
  * it modal. It is common practice to create all widgets that make up
  * the dialog in the constructor and set the caption there. 
  */
MyDialog::MyDialog() :
  QDialog( 0, "Test dialog", true /* modal */ )
{
  // Create a checkbox so that the user has at least something to do. 
  _checkbox = new QCheckBox( "Check me", this );
  _checkbox->setGeometry( 55, 10, 100, 20 );
  
  // Create an OK button. Note how the signal/slot connection
  // completely implements the functionality of this button without
  // any need for user-defined code.
  QPushButton* okbutton = new QPushButton( "OK", this );
  okbutton->setGeometry( 10, 40, 100, 30 );
  QObject::connect( okbutton, SIGNAL( clicked() ),
					this, SLOT( accept() ) );
  
  // Create the cancel button. Similar to the OK button. Again, no
  // user-defined functionality is needed to make this button close
  // the dialog.
  QPushButton* cancelbutton = new QPushButton( "Cancel", this );
  cancelbutton->setGeometry( 120, 40, 100, 30 );
  QObject::connect( cancelbutton, SIGNAL( clicked() ),
					this, SLOT( reject() ) );

  // Set a caption for the dialog. Don't forget this in your dialogs.
  setCaption( "My Test Dialog" );
}


/*
 * This class is a kludge which is necessary because we need an object
 * for the slot that pops up the dialog.
 */
class ClickReceiver : public QObject
{
  Q_OBJECT

public slots:
  void slotShowDialog();
};


/*
 * This is the second interesting method in this program. Note how the
 * document is shown with exec(), the return code checked and how the
 * user settings are only evaluated when the user has pressed OK. You
 * should never evaluate the settings in the dialog when the user has
 * pressed cancel.
 */
void ClickReceiver::slotShowDialog()
{
  // create the dialog
  MyDialog* dialog = new MyDialog();
  // Show it. exec() only returns when the dialog is closed.
  int ret = dialog->exec();
  // Check how the user has closed the dialog. 
  if( ret == QDialog::Accepted )
	{
	  // User pressed OK
	  debug( "User pressed OK; checkbox was %s checked\n", 
		 (dialog->isCheckBoxChecked() ? "" : "not") );
	}
  else
	{
	  // User pressed cancel
	  debug( "User pressed cancel\n" );
	}
}

#include "dialog.moc"

int main( int argc, char* argv[] )
{
  QApplication myapp( argc, argv );
  
  QPushButton* mybutton = new QPushButton( "Click for dialog" );
  mybutton->resize( 150, 30 );

  ClickReceiver clickreceiver;
  QObject::connect( mybutton, SIGNAL( clicked() ),
					&clickreceiver, SLOT( slotShowDialog() ) );

  myapp.setMainWidget( mybutton );
  mybutton->show();
  return myapp.exec();
}
