#include <qapplication.h>
#include <qdragobject.h>
#include <qdropsite.h>
#include <qmultilinedit.h>

class DragDropMultiLineEdit : public QMultiLineEdit, public QDropSite
{
public:
  DragDropMultiLineEdit( QWidget* parent = 0, const char* name = 0 );

protected:
  void mousePressEvent( QMouseEvent* event );
  void mouseMoveEvent( QMouseEvent* event );
  void dragEnterEvent( QDragEnterEvent* event );
  void dropEvent( QDropEvent* event );

private:
  QPoint _presspos;
  int _dragdropbutton;
};


DragDropMultiLineEdit::DragDropMultiLineEdit( QWidget* parent, 
												const char* name ) :
  QMultiLineEdit( parent, name ),
  QDropSite( this )
{
  // determine which button will be the button that invokes dragging
  if( style() == WindowsStyle )
	_dragdropbutton = LeftButton;
  else
	_dragdropbutton = RightButton;
}


void DragDropMultiLineEdit::mousePressEvent( QMouseEvent* event )
{
  // record the position
  if( event->button() == _dragdropbutton )
	_presspos = event->pos();

  QMultiLineEdit::mousePressEvent( event );
}


void DragDropMultiLineEdit::mouseMoveEvent( QMouseEvent* event )
{
  if( event->state() & _dragdropbutton )
	if( QABS( event->pos().x() - _presspos.x() ) >= 3 ||
		QABS( event->pos().y() - _presspos.y() ) >= 3 ) {
	  // Mouse has been dragged for three pixels or more, start
	  // dragging operation
	  QTextDrag* drag = new QTextDrag( text(), this );
	  drag->dragCopy();
	}

  QMultiLineEdit::mouseMoveEvent( event );
}


void DragDropMultiLineEdit::dragEnterEvent( QDragEnterEvent* event )
{
  if( QTextDrag::canDecode( event ) )
	event->accept();
}


void DragDropMultiLineEdit::dropEvent( QDropEvent* event )
{
  QString droptext;
  if( QTextDrag::decode( event, droptext ) ) {
	int row, col;
	cursorPosition( &row, &col );
	insertAt( droptext, row, col );
  }
}


int main( int argc, char* argv[] )
{
  QApplication a( argc, argv );
  
  DragDropMultiLineEdit* edit = new DragDropMultiLineEdit;
  a.setMainWidget( edit );
  edit->show();

  return a.exec();
}
