package ScribbleWindow;

use Qt;
use QWidget;
use QColor;
use QMenuBar;
use QPopupMenu;
use QScrollView;
use QMessageBox;
use ScribbleArea;

use slots 'slotAbout()', 'slotAboutQt()', 'slotColorMenu(int)';
use signals 'colorChanged(QColor)';

@ISA = qw(QWidget);

$COLOR_MENU_ID_BLACK = 0;
$COLOR_MENU_ID_RED = 1;
$COLOR_MENU_ID_BLUE = 2;
$COLOR_MENU_ID_GREEN = 3;
$COLOR_MENU_ID_YELLOW = 4;

#
# Constructor
#
sub new {
  my $class = shift;
  my $self = $class->SUPER::new(@_);

  # The next few lines build up the menubar. We first create the menus
  # one by one and add them afterwards to the menubar.
  my $filemenu = new QPopupMenu; # create a file menu
  $filemenu->insertItem( '&Quit', $qApp, 'quit()' );

  my $colormenu = new QPopupMenu; # create a color menu
  $colormenu->insertItem( 'B&lack', $COLOR_MENU_ID_BLACK );
  $colormenu->insertItem( '&Red', $COLOR_MENU_ID_RED );
  $colormenu->insertItem( '&Blue', $COLOR_MENU_ID_BLUE );
  $colormenu->insertItem( '&Green', $COLOR_MENU_ID_GREEN );
  $colormenu->insertItem( '&Yellow', $COLOR_MENU_ID_YELLOW );
  QObject::connect( $colormenu, 'activated(int)', $self, 'slotColorMenu(int)' );
  
  my $helpmenu = new QPopupMenu; # create a help menu
  $helpmenu->insertItem( '&About QtScribble', $self, 'slotAbout()' );
  $helpmenu->insertItem( 'About &Qt', $self, 'slotAboutQt()' );

  my $menubar = new QMenuBar( $self ); # create a menu bar
  $menubar->insertItem( '&File', $filemenu );
  $menubar->insertItem( '&Color', $colormenu );
  $menubar->insertSeparator();
  $menubar->insertItem( '&Help', $helpmenu );

  # We create a QScrollView and a Scribble Area. The ScribbleArea will
  # be managed by the scroll view.
  my $scrollview = new QScrollView( $self );
  $scrollview->setGeometry( 0, $menubar->height(), $self->width(),
							$self->height() - $menubar->height() );
  my $scribblearea = new ScribbleArea();
  $scribblearea->setGeometry( 0, 0, 1000, 1000 );
  $scrollview->addChild( $scribblearea, 0, 0 );
  QObject::connect( $self, 'colorChanged(QColor)', 
					$scribblearea, 'setColor(QColor)' );

  @$self{ '_menubar', '_filemenu', '_colormenu', '_helpmenu', 
		  '_scrollview', '_scribblearea' } =
			( $menubar, $filemenu, $colormenu, $helpmenu, 
			  $scrollview, $scribblearea );

  return $self;
}


sub DESTROY {
}


sub resizeEvent {
  my $self = shift;
  my $event = shift;

  # When the whole window is resized, we have to rearrange the
  # geometry in the ScrihbleWindow as well. Note that the ScribbleArea
  # does not need to be changed.
  my ( $scrollview, $menubar ) = @$self{ '_scrollview', '_menubar' };
  $scrollview->setGeometry( 0, $menubar->height(),
							$self->width(), 
							$self->height() - $menubar->height() );
}

sub slotAbout {
  my $self = shift;

  QMessageBox::information( $self, 'About QtScribble 3',
							"This is the QtScribble 3 application in Perl\n" .
							"Copyright 1998 by Matthias Kalle Dalheimer\n" );
}


sub slotAboutQt {
  my $self = shift;

  QMessageBox::aboutQt( $self, "About Qt" );
}


sub slotColorMenu {
  my $self = shift;
  my $item = shift;

  if( $item == $COLOR_MENU_ID_BLACK ) {
	emit $self->colorChanged( $black );
  } elsif( $item == $COLOR_MENU_ID_RED ) {
	emit $self->colorChanged( $darkRed );
  } elsif( $item == $COLOR_MENU_ID_BLUE ) {
	emit $self->colorChanged( $darkBlue );
  } elsif( $item == $COLOR_MENU_ID_GREEN ) {
	emit $self->colorChanged( $darkGreen );
  } elsif( $item == $COLOR_MENU_ID_YELLOW ) {
	emit $self->colorChanged( $yellow );
  }
}

