use Qt;
use QApplication;
use QCheckBox;
use QPushButton;

package MyDialog;

@ISA = qw( QDialog );

sub new {
  my $self = shift->SUPER::new( 0, "Test dialog", true );

  

/**
  * The constructor for our dialog. By passing "true" as the third
  * parameter to the constructor of the base class, QDialog, we make
  * it modal. It is common practice to create all widgets that make up
  * the dialog in the constructor and set the caption there.
  */
MyDialog::MyDialog() :
  QDialog( 0, "Test dialog", true /* modal */ )
&lcub;
  // Create a checkbox so that the user has at least something to do.
  checkbox = new QCheckBox( "Check me", this );
  checkbox-&gt;setGeometry( 55, 10, 100, 20 );

  // Create an OK button. Note how the signal/slot connection
  // completely implements the functionality of this button without
  // any need for user-defined code.
  okbutton = new QPushButton( "OK", this );
  okbutton-&gt;setGeometry( 10, 40, 100, 30 );
  QObject::connect( okbutton, SIGNAL( clicked() ),
					this, SLOT( accept() ) );

  // Create the cancel button. Similar to the OK button. Again, no
  // user-defined functionality is needed to make this button close
  // the dialog.
  cancelbutton = new QPushButton( "Cancel", this );
  cancelbutton-&gt;setGeometry( 120, 40, 100, 30 );
  QObject::connect( cancelbutton, SIGNAL( clicked() ),
					this, SLOT( reject() ) );

  // Set a caption for the dialog. Don't forget this in your dialogs.
  setCaption( "My Test Dialog" );
&rcub;


/*
 * The destructor. Here empty, because the widgets are destroyed
 * automatically when the parent is destroyed.
 */
MyDialog::~MyDialog()
&lcub;
&rcub;


/*
 * This class is a kludge which is necessary because we need an object
 * for the slot that pops up the dialog.
 */
class ClickReceiver : public QObject
&lcub;
  Q&lowbar;OBJECT

public slots:
  void slotShowDialog();
&rcub;;


/*
 * This is the second interesting method in this program. Note how the
 * document is shown with exec(), the return code checked and how the
 * user settings are only evaluated when the user has pressed OK. You
 * should never evaluate the settings in the dialog when the user has
 * pressed cancel.
 */
void ClickReceiver::slotShowDialog()
&lcub;
  // create the dialog
  MyDialog* dialog = new MyDialog();
  // Show it. exec() only returns when the dialog is closed.
  int ret = dialog-&gt;exec();
  // Check how the user has closed the dialog.
  if( ret == QDialog::Accepted )
	&lcub;
	  // User pressed OK
	  cout &lt;&lt; "User pressed OK; checkbox was " &lt;&lt;
		(dialog-&gt;isCheckBoxChecked() ? "" : "not") &lt;&lt; " checked&bsol;n";
	&rcub;
  else
	&lcub;
	  // User pressed cancel
	  cout &lt;&lt; "User pressed cancel&bsol;n";
	&rcub;
&rcub;

&num;include "moc&lowbar;dialog.cpp"

int main( int argc, char* argv&lsqb;&rsqb; )
&lcub;
  QApplication myapp( argc, argv );

  QPushButton mybutton( "Click for dialog" );
  mybutton.resize( 150, 30 );

  ClickReceiver clickreceiver;
  QObject::connect( &amp;mybutton, SIGNAL( clicked() ),
					&amp;clickreceiver, SLOT( slotShowDialog() ) );

  myapp.setMainWidget( &amp;mybutton );
  mybutton.show();
  return myapp.exec();
&rcub;
