#include <qnp.h>
#include <qimage.h>
#include <qpainter.h>
#include <qpixmap.h>
#include <qslider.h>
#include <stdio.h>

class ViewWidget : public QWidget
{
  Q_OBJECT

private:
  QPixmap _pm;
  QPixmap _pmScaledRotated;
  int _angle;

public:
  ViewWidget( QWidget* parent = 0, const char* name = 0, WFlags flags = 0 ) :
	QWidget( parent, name, flags ) { _angle = 0; }

  void paintEvent( QPaintEvent* event )
    {
	  QPainter p( this );
	  p.setClipRect( rect() );
	  
	  if ( ( _pm.size() == size() ) && ( _angle == 0 ) ) {
	    p.drawPixmap( 0, 0, _pm );
	  } else {
		QWMatrix m;
		m.scale( (double)width() / _pm.width(),
				 (double)height() / _pm.height() );
		m.rotate( _angle );
		_pmScaledRotated = _pm.xForm( m );
	    p.drawPixmap( 0, 0, _pmScaledRotated );
	  }
    }

    void showImage(const QImage& image)
    {
	  _pm.convertFromImage( image, QPixmap::Color );
	  repaint( FALSE );
    }

public slots:
    void newAngle( int newangle )
	{
	  _angle = newangle;
	  repaint( TRUE );
	}
};


class RotateGIFView : public QNPWidget 
{
public:
  RotateGIFView()
    {
	  _slider = new QSlider( 0, 360, 10, 0, QSlider::Horizontal, this );
	  _slider->show();
	  _view = new ViewWidget( this );
	  _view->show();

	  connect( _slider, SIGNAL( valueChanged( int ) ),
			   _view, SLOT( newAngle( int ) ) );
    }

  void resizeEvent( QResizeEvent* event )
	{
	  _slider->setGeometry( 0, 0, event->size().width(), _slider->height() );
	  _view->setGeometry( 0, _slider->height(), 
						 event->size().width(), 
						 event->size().height()-_slider->height() );
	}
  
  void showImage( const QImage& image )
	{
	  _view->showImage( image );
	}

private:
  QSlider* _slider;
  ViewWidget* _view;
};

class RotateGIFLoader : public QNPInstance 
{
  RotateGIFView* _view;
  QImage _image;
  
public:
  RotateGIFLoader() : 
	_view(0)
    {
    }
  
  QNPWidget* newWindow()
    {
	  _view = new RotateGIFView;
	  imageToGIF();
	  return _view;
    }

  void imageToGIF()
    {
	  if (!_view || _image.isNull()) return;
	  
	  _view->showImage( _image );
	  _image.reset();
    }
  
  bool newStreamCreated( QNPStream*, StreamMode& smode )
    {
	  smode = AsFileOnly;
	  return TRUE;
    }
  
  void streamAsFile( QNPStream*, const char* fname )
    {
	  _image = QImage( fname );
	  if ( _image.isNull() )
	    debug( "Could not convert file: %s\n", fname );
	  imageToGIF();
    }
};


class RotatePlugin : public QNPlugin 
{
public:
  RotatePlugin()
    {
    }

  QNPInstance* newInstance()
    {
	  return new RotateGIFLoader;
    }

  const char* getMIMEDescription() const
    {
	  return "image/gif:gif:GIF Image";
    }

  const char * getPluginNameString() const
    {
	  return "GIF PlQt-based Image Plugin";
    }

    const char * getPluginDescriptionString() const
    {
	return "Let's you view and rotate a GIF image";
    }
};

QNPlugin* QNPlugin::create()
{
    return new RotatePlugin;
}


#include "plugin.moc"
