#include "qkcoordsel.h"
#include <qpainter.h>
#include <qfontmetrics.h>

QKCoordSel::QKCoordSel( QWidget* parent, const char* name, WFlags flags ) :
  QWidget( parent, name, flags )
{
  // default for the ranges is 0-100
  _xrange.setRange( 0, 100 );
  _yrange.setRange( 0, 100 );

  // initial value is (0, 0)
  _xrange.setValue( 0 );
  _yrange.setValue( 0 );

  // calculate the space to set apart for the label
  _labelheight = labelHeight();

  // compute the ratios
  computeRatio();

  // set a cursor for this widget
  setCursor( crossCursor );
}


QKCoordSel::~QKCoordSel()
{
}


int QKCoordSel::xMinValue() const
{
  return _xrange.minValue();
}


int QKCoordSel::xMaxValue() const
{
  return _xrange.maxValue();
}


int QKCoordSel::yMinValue() const
{
  return _yrange.minValue();
}


int QKCoordSel::yMaxValue() const
{
  return _yrange.maxValue();
}


QPoint QKCoordSel::value() const
{
  return QPoint( _xrange.value(), _yrange.value() );
}


void QKCoordSel::setXMinValue( int value )
{
  _xrange.setRange( value, _xrange.maxValue() );
}


void QKCoordSel::setXMaxValue( int value )
{
  _xrange.setRange( _xrange.minValue(), value );
}


void QKCoordSel::setYMinValue( int value )
{
  _yrange.setRange( value, _yrange.maxValue() );
}


void QKCoordSel::setYMaxValue( int value )
{
  _yrange.setRange( _yrange.minValue(), value );
}


void QKCoordSel::setValue( int x, int y )
{
  _xrange.setValue( x );
  _yrange.setValue( y );
}


void QKCoordSel::setValue( const QPoint& point )
{
  _xrange.setValue( point.x() );
  _yrange.setValue( point.y() );
}


void QKCoordSel::resizeEvent( QResizeEvent* )
{
  computeRatio();
}


void QKCoordSel::paintEvent( QPaintEvent* e )
{
  QPainter painter( this );
  painter.drawLine( 0, height()-_labelheight, width(), height()-_labelheight );
  QString coordstext;
  coordstext.sprintf( "%d x %d", _tempvalue.x(), _tempvalue.y() );
  painter.drawText( 0, height(), coordstext );
}


void QKCoordSel::mousePressEvent( QMouseEvent* e )
{
  _tempvalue = mouseToLogical( e->pos() );
  repaint();
}

 
void QKCoordSel::mouseReleaseEvent( QMouseEvent* e )
{
  if( rect().contains( e->pos() ) ) {
	QPoint coords = mouseToLogical( e->pos() );
	emit valueChanged( coords.x(), coords.y() );
	_xrange.setValue( coords.x() );
	_yrange.setValue( coords.y() );
	_tempvalue.setX( coords.x() );
	_tempvalue.setY( coords.y() );
	repaint();
  }
}


void QKCoordSel::mouseMoveEvent( QMouseEvent* e )
{
  QPoint oldtempvalue = _tempvalue;
  _tempvalue = mouseToLogical( e->pos() );
  if( _tempvalue.x() < _xrange.minValue() )
	_tempvalue.setX( _xrange.minValue() );
  if( _tempvalue.y() < _yrange.minValue() )
	_tempvalue.setY( _yrange.minValue() );
  if( _tempvalue.x() > _xrange.maxValue() )
	_tempvalue.setX( _xrange.maxValue() );
  if( _tempvalue.y() > _yrange.maxValue() )
	_tempvalue.setY( _yrange.maxValue() );
  
  repaint();
}


void QKCoordSel::setFont( const QFont& font )
{
  QWidget::setFont( font );
  _labelheight = labelHeight();
  repaint();
}



QPoint QKCoordSel::mouseToLogical( QPoint point )
{
  return QPoint( point.x() * _xratio + _xrange.minValue(),
				 point.y() * _yratio + _yrange.minValue() );
}


int QKCoordSel::labelHeight()
{
  QFontMetrics fm( font() );
  return fm.height();
}


void QKCoordSel::computeRatio()
{
  _xratio = (double)( _xrange.maxValue() - _xrange.minValue() ) / (double)width();
  _yratio = (double)( _yrange.maxValue() - _yrange.minValue() ) / (double)(height()-_labelheight);
}


QSize QKCoordSel::sizeHint() const
{
  return QSize( _xrange.maxValue() - _xrange.minValue(),
				_yrange.maxValue() - _yrange.minValue() + _labelheight );
}
